import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_svg/svg.dart';
import 'package:image_picker/image_picker.dart';
import 'package:reactive_forms/reactive_forms.dart';

import '../../common/custom_scaffold_messages.dart';
import '../../common/loading_painter.dart';
import '../../common/widgets_style.dart';
import '../../successful_sending/succesful_sending_page.dart';
import '../bloc/feedback_bloc.dart';
import 'feedback_page.dart';

class FeedbackView extends StatefulWidget {
  final Data data;
  final Function(
    Map<String, dynamic> form,
  ) onSend;
  final Function(List<XFile>) onFileSend;
  const FeedbackView({
    Key? key,
    required this.data,
    required this.onSend,
    required this.onFileSend,
  }) : super(key: key);

  @override
  State<FeedbackView> createState() => _FeedbackViewState();
}

class _FeedbackViewState extends State<FeedbackView>
    with TickerProviderStateMixin {
  late AnimationController _controller;
  late Animation<double> _animation;
  final Tween<double> turnsTween = Tween<double>(
    begin: 1,
    end: 0,
  );
  void initState() {
    super.initState();
    _controller =
        AnimationController(vsync: this, duration: const Duration(seconds: 2));
    _animation = CurvedAnimation(
      parent: _controller,
      curve: Curves.bounceIn,
    );
    _controller.repeat();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  final form = FormGroup(
    {
      'email': FormControl<String>(
        validators: [
          Validators.required,
          Validators.email,
        ],
      ),
      'topic': FormControl<String>(
        validators: [
          Validators.required,
        ],
      ),
      'problem_description': FormControl<String>(
        validators: [
          Validators.required,
        ],
      ),
    },
  );
  final ImagePicker imagePicker = ImagePicker();
  List<XFile>? imageFileList = [];

  void selectImages() async {
    final List<XFile>? selectedImages = await imagePicker.pickMultiImage();
    if ((imageFileList!.length + selectedImages!.length) <= 5) {
      if (selectedImages.isNotEmpty) {
        imageFileList!.addAll(selectedImages);
        widget.onFileSend(selectedImages);
      }
    } else {
      ScaffoldMessenger.of(context).showSnackBar(CustomScaffoldMessages().show(
          title: 'Максимальное число изображений 5',
          color: widget.data.mainColor));
    }

    setState(() {});
  }

  void _fetchData(BuildContext context) async {
    showDialog(
        barrierDismissible: false,
        context: context,
        builder: (_) {
          return Dialog(
            backgroundColor: Colors.white,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(13),
            ),
            child: Padding(
              padding: const EdgeInsets.symmetric(vertical: 20),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  RotationTransition(
                    turns: turnsTween.animate(_controller),
                    child: CustomPaint(
                      painter: LoadingPainter(color: widget.data.mainColor),
                    ),
                  ),
                  const SizedBox(
                    width: 30,
                  ),
                  const Text(
                    'Отправляем...',
                    style: TextStyle(
                      fontSize: 16,
                      fontWeight: FontWeight.w500,
                      color: Colors.black,
                    ),
                  )
                ],
              ),
            ),
          );
        });
  }

  @override
  Widget build(BuildContext context) {
    return BlocListener<FeedbackBloc, FeedbackState>(
        listener: (BuildContext context, FeedbackState state) {
      state.maybeWhen(
        success: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => SuccesfulSending(
                data: widget.data,
              ),
            ),
          );
        },
        error: () {
          ScaffoldMessenger.of(context).showSnackBar(
            CustomScaffoldMessages().show(
              title: 'При отправки произошла ошибка',
              color: widget.data.mainColor,
            ),
          );
        },
        orElse: () {},
        loading: () {
          _fetchData(context);
        },
      );
    }, child: BlocBuilder<FeedbackBloc, FeedbackState>(
      builder: (BuildContext context, FeedbackState state) {
        return state.maybeWhen(
          orElse: () {
            return ReactiveForm(
              formGroup: form,
              child: Padding(
                padding: const EdgeInsets.symmetric(horizontal: 20),
                child: SingleChildScrollView(
                  child: Column(
                    children: [
                      Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          const SizedBox(
                            height: 28,
                          ),
                          widget.data.imageURL == ''
                              ? const Placeholder(
                                  fallbackHeight: 40,
                                )
                              : Image(
                                  height: 40,
                                  image: NetworkImage(widget.data.imageURL)),
                          const SizedBox(
                            height: 30,
                          ),
                          const Text(
                            'Обратная связь',
                            style: TextStyle(
                              fontSize: 20,
                              fontWeight: FontWeight.w600,
                            ),
                          ),
                          const SizedBox(
                            height: 12,
                          ),
                          Text(
                            widget.data.hint,
                            style: const TextStyle(
                              color: Color(0xFF6C6C6C),
                              fontSize: 14,
                              fontWeight: FontWeight.w400,
                            ),
                          ),
                          const SizedBox(
                            height: 26,
                          ),
                          ReactiveTextField(
                            formControlName: 'email',
                            onSubmitted: () => form.focus('topic'),
                            validationMessages: (control) => {
                              'required': 'Поле должно быть заполнено',
                              'email': 'Введён не корректный email'
                            },
                            decoration:
                                customInputDecoration(labelText: 'Почта'),
                          ),
                          const SizedBox(
                            height: 16,
                          ),
                          ReactiveTextField(
                            formControlName: 'topic',
                            onSubmitted: () =>
                                form.focus('problem_description'),
                            validationMessages: (control) => {
                              'required': 'Поле должно быть заполнено',
                            },
                            decoration: customInputDecoration(
                                labelText: 'Тема проблемы'),
                          ),
                          const SizedBox(
                            height: 16,
                          ),
                          ReactiveTextField(
                            maxLines: 4,
                            formControlName: 'problem_description',
                            validationMessages: (control) => {
                              'required': 'Поле должно быть заполнено',
                            },
                            decoration: customInputDecoration(
                                labelText: 'Описание проблемы'),
                          ),
                          const SizedBox(
                            height: 16,
                          ),
                          if (widget.data.addFile)
                            Column(
                              children: [
                                SizedBox(
                                  height: 55,
                                  child: ElevatedButton(
                                    style: customButtonStyle(
                                        textColor: widget.data.buttonTextColor,
                                        color: widget.data.mainColor),
                                    onPressed: () {
                                      selectImages();
                                    },
                                    child: Row(
                                      children: [
                                        SvgPicture.asset(
                                          'assets/icons/icon_paper_clip.svg',
                                          color: widget.data.buttonTextColor,
                                        ),
                                        const SizedBox(
                                          width: 16,
                                        ),
                                        Text(
                                          'Добавить файлы',
                                          style: TextStyle(
                                            color: widget.data.buttonTextColor,
                                            fontSize: 14,
                                            fontWeight: FontWeight.w500,
                                          ),
                                        ),
                                      ],
                                    ),
                                  ),
                                ),
                                imageFileList!.isNotEmpty
                                    ? GridView.builder(
                                        shrinkWrap: true,
                                        physics:
                                            const NeverScrollableScrollPhysics(),
                                        itemCount: imageFileList!.length,
                                        gridDelegate:
                                            const SliverGridDelegateWithFixedCrossAxisCount(
                                                crossAxisCount: 2),
                                        itemBuilder:
                                            (BuildContext context, int index) {
                                          return Padding(
                                            padding: const EdgeInsets.all(2),
                                            child: ClipRRect(
                                              borderRadius:
                                                  BorderRadius.circular(8.0),
                                              child: Image.file(
                                                File(
                                                    imageFileList![index].path),
                                                fit: BoxFit.cover,
                                              ),
                                            ),
                                          );
                                        })
                                    : const SizedBox()
                              ],
                            )
                        ],
                      ),
                      ReactiveFormConsumer(
                        builder: (context, form, child) {
                          return Column(
                            children: [
                              state.maybeWhen(orElse: () {
                                return form.valid
                                    ? const SizedBox()
                                    : const Padding(
                                        padding: EdgeInsets.only(top: 16),
                                        child: Text(
                                          'Заполните обязательные поля',
                                          style: TextStyle(
                                            color: Color(0xFF6C6C6C),
                                            fontSize: 14,
                                            fontWeight: FontWeight.w400,
                                          ),
                                        ),
                                      );
                              }, fileSend: () {
                                return const Padding(
                                  padding: EdgeInsets.only(top: 16),
                                  child: Text(
                                    'Подождите, идёт загрузка файлов',
                                    style: TextStyle(
                                      color: Color(0xFF6C6C6C),
                                      fontSize: 14,
                                      fontWeight: FontWeight.w400,
                                    ),
                                  ),
                                );
                              }),
                              Padding(
                                padding:
                                    const EdgeInsets.symmetric(vertical: 16),
                                child: SizedBox(
                                  height: 55,
                                  width: double.infinity,
                                  child: ElevatedButton(
                                    style: customButtonStyle(
                                      color: widget.data.mainColor,
                                      textColor: widget.data.buttonTextColor,
                                    ),
                                    onPressed: state.maybeWhen(
                                      orElse: () {
                                        return form.valid
                                            ? () {
                                                widget.onSend(
                                                  form.value,
                                                );
                                              }
                                            : null;
                                      },
                                      fileSend: () {
                                        return null;
                                      },
                                    ),
                                    child: const Text(
                                      'Отправить',
                                    ),
                                  ),
                                ),
                              ),
                            ],
                          );
                        },
                      ),
                    ],
                  ),
                ),
              ),
            );
          },
        );
      },
    ));
  }
}
